@@ -15,11 +15,9 @@ import pydeface.utils as pdu
 import drmaa
 from pathlib import Path
 try:
-    from bidscoin import bids
+    from bidscoin import bidscoin, bids
 except ImportError:
-    import bids             # This should work if bidscoin was not pip-installed
-
-LOGGER = logging.getLogger('bidscoin')
+    import bidscoin, bids             # This should work if bidscoin was not pip-installed
 
 
 def deface(bidsdir: str, pattern: str, subjects: list, output: str, cluster: bool, nativespec: str, kwargs: dict):
@@ -39,7 +37,7 @@ def deface(bidsdir: str, pattern: str, subjects: list, output: str, cluster: boo
     bidsdir = Path(bidsdir).resolve()
 
     # Start logging
-    bids.setup_logging(bidsdir/'code'/'bidscoin'/'deface.log')
+    bidscoin.setup_logging(bidsdir/'code'/'bidscoin'/'deface.log')
     LOGGER.info('')
     LOGGER.info('------------ START deface ------------')
     LOGGER.info(f">>> deface bidsfolder={bidsdir} pattern={pattern} subjects={subjects} output={output}"
@@ -47,7 +45,7 @@ def deface(bidsdir: str, pattern: str, subjects: list, output: str, cluster: boo
 
     # Get the list of subjects
     if not subjects:
-        subjects = bids.lsdirs(bidsdir, 'sub-*')
+        subjects = bidscoin.lsdirs(bidsdir, 'sub-*')
         if not subjects:
             LOGGER.warning(f"No subjects found in: {bidsdir/'sub-*'}")
     else:
@@ -66,7 +64,7 @@ def deface(bidsdir: str, pattern: str, subjects: list, output: str, cluster: boo
         # Loop over bids subject/session-directories
         for n, subject in enumerate(subjects, 1):
 
-            sessions = bids.lsdirs(subject, 'ses-*')
+            sessions = bidscoin.lsdirs(subject, 'ses-*')
             if not sessions:
                 sessions = [subject]
             for session in sessions:
@@ -80,16 +78,16 @@ def deface(bidsdir: str, pattern: str, subjects: list, output: str, cluster: boo
                 for match in sorted([match for match in session.glob(pattern) if '.nii' in match.suffixes]):
 
                     # Construct the output filename and relative path name (used in BIDS)
-                    match_rel = str(match.relative_to(session))
+                    match_rel = str(match.relative_to(session).as_posix())
                     if not output:
                         outputfile     = match
                         outputfile_rel = match_rel
                     elif output == 'derivatives':
                         outputfile     = bidsdir/'derivatives'/'deface'/sub_id/ses_id/match.parent.name/match.name
-                        outputfile_rel = str(outputfile.relative_to(bidsdir))
+                        outputfile_rel = str(outputfile.relative_to(bidsdir).as_posix())
                     else:
                         outputfile     = session/output/match.name
-                        outputfile_rel = str(outputfile.relative_to(session))
+                        outputfile_rel = str(outputfile.relative_to(session).as_posix())
                     outputfile.parent.mkdir(parents=True, exist_ok=True)
 
                     # Deface the image
@@ -136,8 +134,7 @@ def deface(bidsdir: str, pattern: str, subjects: list, output: str, cluster: boo
 
                     # Update the scans.tsv file
                     if (bidsdir/'.bidsignore').is_file():
-                        with (bidsdir/'.bidsignore').open('r') as fid:
-                            bidsignore = fid.read().splitlines()
+                        bidsignore = (bidsdir/'.bidsignore').read_text().splitlines()
                     else:
                         bidsignore = [bids.unknowndatatype + '/']
                     bidsignore.append('derivatives/')
@@ -177,7 +174,7 @@ def main():
                         help="Globlike search pattern (relative to the subject/session folder) to select the images that need to be defaced, e.g. 'anat/*_T1w*'")
     parser.add_argument('-p','--participant_label', type=str, nargs='+',
                         help='Space separated list of sub-# identifiers to be processed (the sub- prefix can be left out). If not specified then all sub-folders in the bidsfolder will be processed')
-    parser.add_argument('-o','--output', type=str, choices=bids.bidsdatatypes + (bids.unknowndatatype, 'derivatives'),
+    parser.add_argument('-o','--output', type=str, choices=bids.bidscoindatatypes + (bids.unknowndatatype, 'derivatives'),
                         help=f"A string that determines where the defaced images are saved. It can be the name of a BIDS datatype folder, such as 'anat', or of the derivatives folder, i.e. 'derivatives'. If output is left empty then the original images are replaced by the defaced images")
     parser.add_argument('-c','--cluster', action='store_true',
                         help='Flag to submit the deface jobs to the high-performance compute (HPC) cluster')
@@ -197,4 +194,8 @@ def main():
 
 
 if __name__ == '__main__':
+    LOGGER = logging.getLogger(f"bidscoin.{Path(__file__).stem}")
     main()
+
+else:
+    LOGGER = logging.getLogger(__name__)
