@@ -12,11 +12,9 @@ import pandas as pd
 from multiecho import combination as me
 from pathlib import Path
 try:
-    from bidscoin import bids
+    from bidscoin import bidscoin, bids
 except ImportError:
-    import bids             # This should work if bidscoin was not pip-installed
-
-LOGGER = logging.getLogger('bidscoin')
+    import bidscoin, bids             # This should work if bidscoin was not pip-installed
 
 
 def echocombine(bidsdir: str, pattern: str, subjects: list, output: str, algorithm: str, weights: list, force: bool=False):
@@ -36,7 +34,7 @@ def echocombine(bidsdir: str, pattern: str, subjects: list, output: str, algorit
     bidsdir = Path(bidsdir).resolve()
 
     # Start logging
-    bids.setup_logging(bidsdir/'code'/'bidscoin'/'echocombine.log')
+    bidscoin.setup_logging(bidsdir/'code'/'bidscoin'/'echocombine.log')
     LOGGER.info('')
     LOGGER.info(f"--------- START echocombine ---------")
     LOGGER.info(f">>> echocombine bidsfolder={bidsdir} pattern={pattern} subjects={subjects} output={output}"
@@ -47,7 +45,7 @@ def echocombine(bidsdir: str, pattern: str, subjects: list, output: str, algorit
 
     # Get the list of subjects
     if not subjects:
-        subjects = bids.lsdirs(bidsdir, 'sub-*')
+        subjects = bidscoin.lsdirs(bidsdir, 'sub-*')
         if not subjects:
             LOGGER.warning(f"No subjects found in: {bidsdir/'sub-*'}")
     else:
@@ -57,7 +55,7 @@ def echocombine(bidsdir: str, pattern: str, subjects: list, output: str, algorit
     # Loop over bids subject/session-directories
     for n, subject in enumerate(subjects, 1):
 
-        sessions = bids.lsdirs(subject, 'ses-*')
+        sessions = bidscoin.lsdirs(subject, 'ses-*')
         if not sessions:
             sessions = [subject]
         for session in sessions:
@@ -75,7 +73,7 @@ def echocombine(bidsdir: str, pattern: str, subjects: list, output: str, algorit
                 echonr    = bids.get_bidsvalue(match, 'echo')
                 mepattern = bids.get_bidsvalue(match, 'echo', '*')
                 echos     = sorted(match.parent.glob(mepattern.name))
-                newechos  = [echo.parents[1] / bids.unknowndatatype / echo.name for echo in echos]
+                newechos  = [echo.parents[1]/bids.unknowndatatype/echo.name for echo in echos]
                 if not echonr:
                     LOGGER.warning(f"No 'echo' key-value pair found in the filename, skipping: {match}")
                     continue
@@ -97,7 +95,7 @@ def echocombine(bidsdir: str, pattern: str, subjects: list, output: str, algorit
                     continue
 
                 # Combine the multi-echo images
-                me.me_combine(mepattern, cefile, algorithm, weights, saveweights=False, logger=LOGGER.name)
+                me.me_combine(mepattern, cefile, algorithm, weights, saveweights=False)
 
                 # (Re)move the original multi-echo images
                 if not output:
@@ -113,57 +111,63 @@ def echocombine(bidsdir: str, pattern: str, subjects: list, output: str, algorit
                         echo.with_suffix('').with_suffix('.json').unlink()
 
                 # Construct relative path names as they are used in BIDS
-                echos_rel    = [str(echo.relative_to(session)) for echo in echos]
-                newechos_rel = [str(echo.relative_to(session)) for echo in newechos]
-                if output != 'derivatives':
-                    cefile_rel = str(cefile.relative_to(session))
+                oldechos_rel = [str(echo.relative_to(session).as_posix()) for echo in echos]
+                newechos_rel = [str(echo.relative_to(session).as_posix()) for echo in echos + newechos if echo.is_file()]
+                if output == 'derivatives':
+                    cefile_rel = ''                 # This doesn't work for IntendedFor in BIDS :-(
+                else:
+                    cefile_rel = str(cefile.relative_to(session).as_posix())
 
                 # Update the IntendedFor fields in the fieldmap sidecar files (i.e. remove the old echos, add the echo-combined image and, optionally, the new echos)
                 if output != 'derivatives' and (session/'fmap').is_dir():
                     for fmap in (session/'fmap').glob('*.json'):
                         with fmap.open('r') as fmap_fid:
-                            fmap_data = json.load(fmap_fid)
-                        if 'IntendedFor' in fmap_data:
-                            intendedfor = fmap_data['IntendedFor']
-                            if type(intendedfor)==str:
-                                intendedfor = [intendedfor]
-                            if echos_rel[0] in intendedfor:
-                                LOGGER.info(f"Updating 'IntendedFor' to {cefile_rel} in {fmap}")
-                                if not output:
-                                    intendedfor = [file for file in intendedfor if not file in echos_rel] + [cefile_rel] + [newecho for newecho in newechos_rel]
-                                elif output == datatype:
-                                    intendedfor = [file for file in intendedfor if not file in echos_rel] + [cefile_rel]
-                                else:
-                                    intendedfor = intendedfor + [cefile_rel]
-                                fmap_data['IntendedFor'] = intendedfor
-                                with fmap.open('w') as fmap_fid:
-                                    json.dump(fmap_data, fmap_fid, indent=4)
+                            metadata = json.load(fmap_fid)
+                        intendedfor = metadata.get('IntendedFor', [])
+                        if isinstance(intendedfor, str):
+                            intendedfor = [intendedfor]
+                        if oldechos_rel[0] in intendedfor:
+                            LOGGER.info(f"Updating 'IntendedFor' in {fmap}")
+                            metadata['IntendedFor'] = [file for file in intendedfor if file not in oldechos_rel] + newechos_rel + list(filter(None,[cefile_rel]))
+                            with fmap.open('w') as fmap_fid:
+                                json.dump(metadata, fmap_fid, indent=4)
 
                 # Update the scans.tsv file
                 if (bidsdir/'.bidsignore').is_file():
-                    with (bidsdir/'.bidsignore').open('r') as fid:
-                        bidsignore = fid.read().splitlines()
+                    bidsignore = (bidsdir/'.bidsignore').read_text().splitlines()
                 else:
                     bidsignore = [bids.unknowndatatype + '/']
-                bidsignore.append('derivatives/')
                 scans_tsv = session/f"{sub_id}{bids.add_prefix('_',ses_id)}_scans.tsv"
-                if output+'/' not in bidsignore and scans_tsv.is_file():
+                if scans_tsv.is_file():
 
-                    LOGGER.info(f"Adding {cefile_rel} to {scans_tsv}")
-                    scans_table                 = pd.read_csv(scans_tsv, sep='\t', index_col='filename')
-                    scans_table.loc[cefile_rel] = scans_table.loc[echos_rel[0]]
+                    scans_table = pd.read_csv(scans_tsv, sep='\t', index_col='filename')
+                    if oldechos_rel[0] in scans_table.index:
+                        scans_table.loc['oldrow'] = scans_table.loc[oldechos_rel[0]]
+                    elif 'acq_time' in scans_table:
+                        with cefile.with_suffix('').with_suffix('.json').open('r') as fid:
+                            metadata = json.load(fid)
+                        scans_table.loc['oldrow', 'acq_time'] = metadata.get('AcquisitionTime')
+                    else:
+                        scans_table.loc['oldrow'] = None
 
-                    for echo, newecho in zip(echos_rel, newechos_rel):
-                        if not output:
-                            LOGGER.info(f"Updating {echo} -> {newecho} in {scans_tsv}")
-                            scans_table.loc[newecho] = scans_table.loc[echo]
-                            scans_table.drop(echo, inplace=True)
-                        elif output == datatype:
-                            LOGGER.info(f"Removing {echo} from {scans_tsv}")
+                    if output+'/' not in bidsignore + ['derivatives/'] and cefile.parent.name in bids.bidsdatatypes:
+                        LOGGER.info(f"Adding '{cefile_rel}' to '{scans_tsv}'")
+                        scans_table.loc[cefile_rel] = scans_table.loc['oldrow']
+
+                    for echo in oldechos_rel + newechos_rel:
+                        if echo in scans_table.index and not (session/echo).is_file():
+                            LOGGER.info(f"Removing '{echo}' from '{scans_tsv}'")
                             scans_table.drop(echo, inplace=True)
+                        elif echo not in scans_table.index and (session/echo).is_file() and echo.split('/')[0] in bids.bidsdatatypes:
+                            LOGGER.info(f"Adding '{echo}' to '{scans_tsv}'")
+                            scans_table.loc[echo] = scans_table.loc['oldrow']       # NB: Assuming that the echo-rows are all identical
 
+                    scans_table.drop('oldrow', inplace=True)
                     scans_table.sort_values(by=['acq_time','filename'], inplace=True)
-                    scans_table.to_csv(scans_tsv, sep='\t', encoding='utf-8')
+                    scans_table.replace('','n/a').to_csv(scans_tsv, sep='\t', encoding='utf-8', na_rep='n/a')
+                    for scan in scans_table.index:
+                        if not (session/scan).is_file():
+                            LOGGER.warning(f"Found non-existent file '{scan}' in '{scans_tsv}'")
 
     LOGGER.info('-------------- FINISHED! -------------')
     LOGGER.info('')
@@ -188,7 +192,7 @@ def main():
                         help="Globlike recursive search pattern (relative to the subject/session folder) to select the first echo of the images that need to be combined, e.g. '*task-*echo-1*'")
     parser.add_argument('-p','--participant_label', type=str, nargs='+',
                         help='Space separated list of sub-# identifiers to be processed (the sub- prefix can be left out). If not specified then all sub-folders in the bidsfolder will be processed')
-    parser.add_argument('-o','--output', type=str, choices=bids.bidsdatatypes + (bids.unknowndatatype, 'derivatives'), default='',
+    parser.add_argument('-o','--output', type=str, choices=bids.bidscoindatatypes + (bids.unknowndatatype, 'derivatives'), default='',
                         help=f"A string that determines where the output is saved. It can be the name of a BIDS datatype folder, such as 'func', or of the derivatives folder, i.e. 'derivatives'. If output = [the name of the input datatype folder] then the original echo images are replaced by one combined image. If output is left empty then the combined image is saved in the input datatype folder and the original echo images are moved to the {bids.unknowndatatype} folder")
     parser.add_argument('-a','--algorithm', choices=['PAID', 'TE', 'average'], default='TE',
                         help='Combination algorithm')
@@ -208,4 +212,8 @@ def main():
 
 
 if __name__ == '__main__':
+    LOGGER = logging.getLogger(f"bidscoin.{Path(__file__).stem}")
     main()
+
+else:
+    LOGGER = logging.getLogger(__name__)
