@@ -13,8 +13,9 @@ import struct
 import logging
 import numpy as np
 import matplotlib.pyplot as plt
+import dateutil.parser
 from typing import Union
-from pydicom import dcmread, tag
+from pydicom import dcmread, tag, multival
 from pathlib import Path
 
 # Defaults
@@ -22,7 +23,7 @@ LOGVERSION = 'EJA_1'                # This is the file format this function expe
 FREQ       = 1 / 2.5E-3             # Sampling frequency (the unit of SIEMENS ticks is 2.5 ms)
 
 # Set-up logging
-LOGGER = logging.getLogger('physio')
+LOGGER = logging.getLogger(__name__)
 
 
 def readparsefile(fn: Union[bytes,Path], logdatatype: str, firsttime: int=0, expectedsamples: int=0) -> tuple:
@@ -33,7 +34,7 @@ def readparsefile(fn: Union[bytes,Path], logdatatype: str, firsttime: int=0, exp
     :param logdatatype:     Datatype that is extracted, e.g. 'ECG', 'RESP', 'PULS' or 'EXT'. Additional meta data is extracted if 'ACQUISITION_INFO'
     :param firsttime:       Value from readparsefile('ACQUISITION_INFO', ..) that has to be passed for parsing other logdatatypes
     :param expectedsamples: Number of samples of the parsed traces
-    :return:                traces, UUID[, nrslices, nrvolumes, firsttime, lasttime, nrechoes] ([..] if logdatatype=='ACQUISITION_INFO')
+    :return:                traces, UUID[, scandate, nrslices, nrvolumes, firsttime, lasttime, nrechoes] ([..] if logdatatype=='ACQUISITION_INFO')
     """
 
     # Echoes parameter was not added until R015a, so prefill a default value for compatibility with older data
@@ -45,8 +46,7 @@ def readparsefile(fn: Union[bytes,Path], logdatatype: str, firsttime: int=0, exp
         lines = fn.decode('UTF-8').splitlines()
     elif isinstance(fn, Path):                          # Otherwise, fn must be a filename
         LOGGER.info(f"Reading physio log-file: {fn}")
-        with fn.open('r') as fid:
-            lines = fid.read().splitlines()
+        lines = fn.read_text().splitlines()
     else:
         LOGGER.error(f"Wrong input {fn}: {type(fn)}"); raise FileNotFoundError(fn)
 
@@ -94,6 +94,8 @@ def readparsefile(fn: Union[bytes,Path], logdatatype: str, firsttime: int=0, exp
                 if logdatatype != 'ACQUISITION_INFO':
                     LOGGER.error(f"Invalid [{varname}] parameter found"); raise ValueError(varname)
                 nrechoes = int(value)
+            if varname == 'ScanDate':
+                scandate = value
 
         else:
 
@@ -162,7 +164,7 @@ def readparsefile(fn: Union[bytes,Path], logdatatype: str, firsttime: int=0, exp
 
     if logdatatype == 'ACQUISITION_INFO':
         traces = traces - firsttime
-        return traces, UUID, nrslices, nrvolumes, firsttime, lasttime, nrechoes
+        return traces, UUID, scandate, nrslices, nrvolumes, firsttime, lasttime, nrechoes
     else:
         return traces, UUID
 
@@ -177,6 +179,7 @@ def readphysio(fn: Union[str,Path]) -> dict:
     returns active (i.e. non-zero) physio traces for ECG1, ECG2, ECG3, ECG4, RESP, PULS, EXT/EXT1 and EXT2 signals:
 
     physio['UUID']:     Universally unique identifier string for this measurement
+    physio['ScanDate']: The date/time string of the start of the data acquisition
     physio['Freq']:     Sampling frequency in Hz (= 1/clock-tick; The unit of time is clock-ticks, which normally is 2.5 ms)
     physio['SliceMap']: [2 x Volumes x Slices]     [1:2,:,:] = start & finish time stamp of each volume/slice
     physio['ACQ']:      [length = nr of samples]   True if acquisition is active at this time; False if not
@@ -188,12 +191,14 @@ def readphysio(fn: Union[str,Path]) -> dict:
     physio['PULS']:     [length = nr of samples]   PULS signal on this channel
     physio['EXT1']:     [length = nr of samples]   True if EXT/EXT1 signal detected; False if not
     physio['EXT2']:     [length = nr of samples]   True if EXT2 signal detected; False if not
+    physio['Meta']:     Dictionary with additional meta-data from the DICOM header (e.g. 'SeriesNumber')
 
     :param fn:  Either the fullpath of the DICOM file or the basename of the PHYSIO logfiles (fullpath without suffix and file extension, e.g. 'foo/bar/Physio_DATE_TIME_UUID')
     :return:    The active (non-zero) physio traces for ECG1, ECG2, ECG3, ECG4, RESP, PULS, EXT1 and EXT2 signals
     """
 
     foundECG = foundRESP = foundPULS = foundEXT = False
+    metadata = {}
 
     # Check input
     fn = Path(fn).resolve()
@@ -203,7 +208,7 @@ def readphysio(fn: Union[str,Path]) -> dict:
         LOGGER.info(f"Reading physio DICOM file: {fn}")
         dicomdata    = dcmread(fn, force=True)          # The DICM tag may be missing for anonymized DICOM files
         manufacturer = dicomdata.get('Manufacturer')
-        physiotag    = tag.Tag('7fe1', '1010')
+        physiotag    = tag.Tag(0x7fe1, 0x1010)          # A private Siemens tag
         if manufacturer and manufacturer != 'SIEMENS':
             LOGGER.warning(f"Unsupported manufacturer: '{manufacturer}', this function is designed for SIEMENS advanced physiological logging data")
         if dicomdata.get('ImageType')==['ORIGINAL','PRIMARY','RAWDATA','PHYSIO'] and dicomdata.get(physiotag).private_creator=='SIEMENS CSA NON-IMAGE':
@@ -240,6 +245,27 @@ def readphysio(fn: Union[str,Path]) -> dict:
         else:
             LOGGER.error(f"{fn} is not a valid DICOM format file"); raise RuntimeError(f"Invalid DICOM: {fn}")
 
+        # Add some (BIDS) meta-data from the DICOM header
+        for key in ('Modality',
+                    'Manufacturer',
+                    'ManufacturerModelName',
+                    'StationName',
+                    'DeviceSerialNumber',
+                    'SoftwareVersions',
+                    'InstitutionName',
+                    'InstitutionAddress',
+                    'PatientPosition',
+                    'BodyPartExamined',
+                    'ImageType',
+                    'ProtocolName',
+                    'SeriesDescription',
+                    'SeriesNumber',
+                    'AcquisitionNumber'):
+            val = dicomdata.get(key, '')
+            if isinstance(val, multival.MultiValue):
+                val = list(val)
+            metadata[key] = val
+
     # If we don't have an encoded DICOM, check what text log files we have
     else:
         fnINFO = fn.with_name(fn.name + '_Info.log')
@@ -258,7 +284,7 @@ def readphysio(fn: Union[str,Path]) -> dict:
         LOGGER.error('No data files (ECG/RESP/PULS/EXT) found'); raise FileNotFoundError(fn)
 
     # Read in and/or parse the data
-    slicemap, UUID1, nrslices, nrvolumes, firsttime, lasttime, nrechoes = readparsefile(fnINFO, 'ACQUISITION_INFO')
+    slicemap, UUID1, scandate, nrslices, nrvolumes, firsttime, lasttime, nrechoes = readparsefile(fnINFO, 'ACQUISITION_INFO')
     if lasttime <= firsttime:
         LOGGER.error(f"Last timestamp {lasttime} is not greater than first timestamp {firsttime}, aborting..."); raise ValueError(lasttime)
     actualsamples   = lasttime - firsttime + 1
@@ -301,8 +327,10 @@ def readphysio(fn: Union[str,Path]) -> dict:
     # Only return active (nonzero) physio traces
     physio             = dict()
     physio['UUID']     = UUID1
+    physio['ScanDate'] = dateutil.parser.parse(scandate, fuzzy=True).strftime('%Y-%m-%dT%H:%M:%S')
     physio['Freq']     = FREQ
     physio['SliceMap'] = slicemap
+    physio['Meta']     = metadata
     physio['ACQ']      = ACQ[:,0]
     if foundECG and ECG.any():
         if sum(ECG[:,0]): physio['ECG1'] = ECG[:,0]
@@ -320,34 +348,37 @@ def readphysio(fn: Union[str,Path]) -> dict:
     return physio
 
 
-def physio2tsv(physio: dict, bidsname: Union[str,Path]):
+def physio2tsv(physio: dict, tsvfile: Union[str, Path]):
     """
-    Saves the physiological traces to a BIDS-compliant [bidsname].tsv.nii and a [bidsname].log file
+    Saves the physiological traces to a BIDS-compliant [tsvfile].tsv.nii and a [tsvfile].log file
 
-    :param physio:      Physio data dictionary from readphysio
-    :param bidsname:    Fullpath name of the bids output .tsv.nii/.json file
+    :param physio:  Physio data dictionary from readphysio
+    :param tsvfile: Fullpath name of the (BIDS) output .tsv.gz/.json file
     :return:
     """
 
     # Check input
-    bidsname = Path(bidsname).resolve()
+    tsvfile = Path(tsvfile).resolve()
 
     # Set the clock at zero at the start of the MRI acquisition
     starttime = -physio['ACQ'].nonzero()[0][0] / physio['Freq']     # Assumes that the physiological acquisition always starts before the MRI acquisition
 
     # Add each trace to a data table and save the table as a BIDS-compliant gzipped tsv file
-    physiotable = pd.DataFrame(columns=[key for key in physio if key not in ('UUID','Freq','SliceMap','ACQ')])
+    physiotable = pd.DataFrame(columns=[key for key in physio if key not in ('UUID','ScanDate','Freq','SliceMap','ACQ','Meta')])
     for key in physiotable.columns:
         physiotable[key] = physio[key]
-    LOGGER.info(f"Writing physiological traces to: '{bidsname.with_suffix('.tsv.gz')}'")
-    physiotable.to_csv(bidsname.with_suffix('.tsv.gz'), header=False, index=False, sep='\t', compression='infer')
+    LOGGER.info(f"Writing physiological traces to: '{tsvfile.with_suffix('.tsv.gz')}'")
+    physiotable.to_csv(tsvfile.with_suffix('.tsv.gz'), header=False, index=False, sep='\t', compression='infer')
 
     # Write a json side-car file
-    with bidsname.with_suffix('.json').open('w') as json_fid:
-        json.dump({'SamplingFrequency': physio['Freq'],
-                   'StartTime': starttime,
-                   'Columns': physiotable.columns.to_list()},
-                  json_fid, indent=4)
+    version = (Path(__file__).parent/'version.txt').read_text().strip()
+    physio['Meta']['SamplingFrequency'] = physio['Freq']
+    physio['Meta']['StartTime']         = starttime
+    physio['Meta']['AcquisitionTime']   = physio['ScanDate']
+    physio['Meta']['Columns']           = physiotable.columns.to_list()
+    physio['Meta']['GeneratedBy']       = [{'name':'BIDScoin', 'Version':version, 'CodeURL':'https://github.com/Donders-Institute/bidscoin'}]
+    with tsvfile.with_suffix('.json').open('w') as json_fid:
+        json.dump(physio['Meta'], json_fid, indent=4)
 
 
 def plotphysio(physio:dict, showsamples: int=1000):
