@@ -11,9 +11,9 @@ import re
 import warnings
 from pathlib import Path
 try:
-    from bidscoin import bids
+    from bidscoin import bidscoin, bids
 except ImportError:
-    import bids         # This should work if bidscoin was not pip-installed
+    import bidscoin, bids         # This should work if bidscoin was not pip-installed
 
 
 def rawmapper(rawfolder, outfolder: Path=Path(), sessions: tuple=(), rename: bool=False, dicomfield: tuple=('PatientComments',), wildcard: str='*', subprefix: str='sub-', sesprefix: str='ses-', dryrun: bool=False) -> None:
@@ -41,11 +41,9 @@ def rawmapper(rawfolder, outfolder: Path=Path(), sessions: tuple=(), rename: boo
     mapperfile = outfolder/f"rawmapper_{'_'.join(dicomfield)}.tsv"
     if not dryrun:
         if rename and not mapperfile.is_file():     # Write the header once
-            with mapperfile.open('w') as fid:
-                fid.write('subid\tsesid\tnewsubid\tnewsesid\n')
-        else:     # Write the header once
-            with mapperfile.open('w') as fid:
-                fid.write('subid\tsesid\tseriesname\t{}\n'.format('\t'.join(dicomfield)))
+            mapperfile.write_text('subid\tsesid\tnewsubid\tnewsesid\n')
+        else:                                       # Write the header once
+            mapperfile.write_text('subid\tsesid\tseriesname\t{}\n'.format('\t'.join(dicomfield)))
 
     # Map the sessions in the sourcefolder
     if not sessions:
@@ -65,7 +63,7 @@ def rawmapper(rawfolder, outfolder: Path=Path(), sessions: tuple=(), rename: boo
         sesid = sesid.replace('ses-', sesprefix)
 
         # Parse the new subject and session identifiers from the dicomfield
-        series = bids.lsdirs(session, wildcard)
+        series = bidscoin.lsdirs(session, wildcard)
         if not series:
             series = ''
             dcmval = ''
